//+------------------------------+
//|   StopLoss Manager           |
//+==============================+
//|
//|   Rev Date: 2010.05.24
//|
//|   Contains the following Functions:
//|
//|   StopLossPricebyPercent()
//|   StopLossPricebyATR()
//|
//+------------------------------+

#property copyright "1005phillip"
#include <stderror.mqh>
#include <stdlib.mqh>

//+------------------------------+
//|   StopLossPricebyPercent()   |
//|==============================|
//|   double StopLossPricebyPercent(double StopLossPercent,int CurrentOrderType,bool verbose=false)
//|
//|   Computes the stoploss price based on the input stoplosspercent value as well as the ordertype of the impending trade.
//|   Returns a double value representing the stoploss price for the order if possible, otherwise, it returns a zero value.
//|
//|   Parameters:
//|
//|      StopLossPercent   -  A percent value of the Bid or Ask price at which point the stop is to be placed.
//|      CurrentOrderType  -  Is the impending order going to be an OP_BUY or OP_SELL.
//|      verbose           -  Determines whether Print messages are committed to the experts log. By default, print messages are disabled.
//|
//|   Sample:
//|
//|      StopLossPercent=2./100.;   // setting the stoplosspercent at a fixed 2% level of the financial instrument's price
//|      CurrentStopLossPrice=StopLossPricebyPercent(StopLossPercent,OP_BUY)  // compute the stoploss price for a Buy order, no verbose comments
//|      Print("StopLossPrice = ",CurrentStopLossPrice," and StopLoss in Pips = ",(CurrentStopLossPrice-Bid)/MarketInfo(Symbol(),MODE_POINT));
//|
//+------------------------------+
double StopLossPricebyPercent(double StopLossPercent,int CurrentOrderType,bool verbose=false)
   {  // StopLossPrice body start
   double   CalculatedStopLossPrice=0.;
   
   switch(CurrentOrderType) // Determine StopLossPrice based on the intended OrderType of the trade
      {
      case OP_BUY    :  CalculatedStopLossPrice=NormalizeDouble(Ask-MathRound(StopLossPercent*Ask/MarketInfo(Symbol(),MODE_POINT))*MarketInfo(Symbol(),MODE_POINT),MarketInfo(Symbol(),MODE_DIGITS));
                        if(verbose==true) Print("Ask = ",Ask,", StopLossPrice = ",CalculatedStopLossPrice," and StopLoss in Pips = ",(Ask-CalculatedStopLossPrice)/MarketInfo(Symbol(),MODE_POINT));  break;
      case OP_SELL   :  CalculatedStopLossPrice=NormalizeDouble(Bid+MathRound(StopLossPercent*Bid/MarketInfo(Symbol(),MODE_POINT))*MarketInfo(Symbol(),MODE_POINT),MarketInfo(Symbol(),MODE_DIGITS));
                        if(verbose==true) Print("Bid = ",Bid,", StopLossPrice = ",CalculatedStopLossPrice," and StopLoss in Pips = ",(CalculatedStopLossPrice-Bid)/MarketInfo(Symbol(),MODE_POINT));  break;
      default        :  Print("Error encountered in the SWITCH routine for calculating the StopLossPrice"); // The expression did not generate a case value
      }
   
   return(CalculatedStopLossPrice);
   
   }  // StopLossPrice body end

// Function End

//+------------------------------+
//|   StopLossPricebyATR()       |
//|==============================|
//|   double StopLossPricebyATR(int ATRTimeFrame,int ATRPeriod,int CurrentOrderType,bool verbose=false)
//|
//|   Computes the stoploss price based on the input ATR timeframe and period as well as the ordertype of the trade.
//|   Returns a double value representing the stoploss price for the order if possible, otherwise, it returns a zero value.
//|
//|   Parameters:
//|
//|      ATRTimeFrame      -  Timeframe. It can be any of Timeframe enumeration values. 0 means the current chart timeframe.
//|      ATRPeriod         -  Averaging period for calculation.
//|      CurrentOrderType  -  Is the impending order going to be an OP_BUY or OP_SELL.
//|      verbose           -  Determines whether Print messages are committed to the experts log. By default, print messages are disabled.
//|
//|   Sample:
//|
//|      int CurrentATRTimeFrame=1440;    // setting the ATR timeframe to the daily chart
//|      int CurrentATRPeriod=5;          // setting the ATR averaging period to 5, representing one market week on a daily rolling basis
//|      double CurrentStopLossPrice=StopLossPricebyATR(CurrentATRTimeFrame,CurrentATRPeriod,OP_BUY)  // compute the stoploss price based on ATR for a Buy order, no verbose comments
//|      Print("StopLossPrice = ",CurrentStopLossPrice," and StopLoss in Pips = ",(CurrentStopLossPrice-Bid)/MarketInfo(Symbol(),MODE_POINT));
//|
//+------------------------------+
double StopLossPricebyATR(int ATRTimeFrame,int ATRPeriod,int CurrentOrderType,bool verbose=false)
   {  // StopLossPrice body start
   double         CalculatedStopLossPrice=0.;
   static double  ATRArray[3],ATRArrayMax=0.;
   static int     BarOpenATRTimeFrame;
   
   // Verify there are enough bars present in the requested chart timeframe to enable computation of the requested indicator's period
   if(iBars(Symbol(),ATRTimeFrame)<ATRPeriod+1)
      {
      Print("Not enough bars in the specified chart for computing stoploss by ATR");
      Print("Current timedate = ",TimeToStr(TimeCurrent())," and bar count is ",iBars(Symbol(),ATRTimeFrame));
      return(0);
      }
   
   // Determine if a new bar has occurred in the requested chart since the mos recent computation of the requested indicators
   if(iTime(Symbol(),ATRTimeFrame,0)!=BarOpenATRTimeFrame)  // Current bar's open timestamp does not equal the one in memory, recalc the indicator values
      {
      ATRArray[0]=iATR(NULL,ATRTimeFrame,ATRPeriod,0);      // Compute the Longer term ATR
      ATRArray[1]=iATR(NULL,ATRTimeFrame,1,0);              // Compute the ATR for current candle to capture immediate market volatility
      ATRArray[2]=iATR(NULL,ATRTimeFrame,1,1);              // Compute the ATR for preceding candle to capture immediate market volatility
      
      ATRArrayMax=ATRArray[ArrayMaximum(ATRArray)];         // Compute the maximum ATR value of the array
      
      BarOpenATRTimeFrame=iTime(Symbol(),ATRTimeFrame,0);   // Remember the current bar open time for future reference
      }
   
   switch(CurrentOrderType) // Determine StopLossPrice based on the intended OrderType of the trade
      {
      case OP_BUY    :  CalculatedStopLossPrice=NormalizeDouble(Ask-ATRArrayMax,MarketInfo(Symbol(),MODE_DIGITS));
                        if(verbose==true)
                           {
                           Print("Ask = ",Ask,", StopLossPrice = ",CalculatedStopLossPrice," and StopLoss in Pips = ",(Ask-CalculatedStopLossPrice)/MarketInfo(Symbol(),MODE_POINT));
                           Print("ATRArray[0] = ",ATRArray[0],", ATRArray[1] = ",ATRArray[1],", ATRArray[2] = ",ATRArray[2]," and ATRArrayMax = ",ATRArrayMax);
                           }
                        break;
      case OP_SELL   :  CalculatedStopLossPrice=NormalizeDouble(Bid+ATRArrayMax,MarketInfo(Symbol(),MODE_DIGITS));
                        if(verbose==true)
                           {
                           Print("Bid = ",Bid,", StopLossPrice = ",CalculatedStopLossPrice," and StopLoss in Pips = ",(CalculatedStopLossPrice-Bid)/MarketInfo(Symbol(),MODE_POINT));
                           Print("ATRArray[0] = ",ATRArray[0],", ATRArray[1] = ",ATRArray[1],", ATRArray[2] = ",ATRArray[2]," and ATRArrayMax = ",ATRArrayMax);
                           }
                        break;
      default        :  Print("Error encountered in the SWITCH routine for calculating the StopLossPrice by ATR"); // The expression did not generate a case value
      }
   
   return(CalculatedStopLossPrice);
   
   }  // StopLossPrice body end

// Function End

// Program End

/*
   +------------------------------------------------------------------+
   |                                                                  |
   |   Revision History                                               |
   |                                                                  |
   +------------------------------------------------------------------+

 Objective: This include file will contain the routines necessary for computing stoplossprice based on a stoplosspercent stops method as well as compute the equity at risk for a
            given trade position size (lotsize) and the stops that will be used in the trade.
            
 Future Work:  Add fractal-based stoploss call function
            
 Worklog:
   
   May 20   Upgrading the ATR stoploss calculator to only compute the ATR values if a new bar has incremented in the timeframe chart requested
   
   May 20   Creating StopLossPricebyPercent() and StopLossPricebyATR()
   
*/


